# Test that when the Students t-test is valid, Pointbiserial is as well. 

# When pointbiserial is valid, Students t shuold be, too -- they have the same constraints

# Add test about what happens when have different sizes in samples in pearson r